import 'dart:async';
import 'dart:math';
import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:flutter_webrtc/flutter_webrtc.dart';
import 'package:the_paint/src/generated/client.pb.dart';
import 'package:the_paint/src/square/square.dart';

import '../connection/ws.dart';

class WebRTCSquareData extends SquereData {
  final Resolver resolver = WebRTCChannels();
  final StreamController<MonitorReply> streamChannel =
      StreamController<MonitorReply>();
      
  Duplex? _dc;

  WebRTCSquareData(super.tile);

  @override
  Stream<MonitorReply> getSteam() {
    resolver.dataChannel(super.tile).then((dc) {
      _dc = dc;
      streamChannel.addStream(dc.getStream().map((event) {
        var point = Point(1, 1);
        var color = Colors.blueGrey;
        return MonitorReply(points: [
          MonitorPoint(
              point: BPoint(x: point.x, y: point.y), color: fromColor(color))
        ]);
      }));
    });
    return streamChannel.stream;
  }

  @override
  void paint(Point<int> point, Color color) {
    if (_dc == null ) {
      return;
    }
    _dc!.send(serialize(point, color));
  }

  RTCDataChannelMessage serialize(Point<int> point, Color color) {
    final data = MonitorPoint(
      point: BPoint(x: point.x, y: point.y),
      color: fromColor(color),
    ).writeToBuffer();
    return RTCDataChannelMessage.fromBinary(data);
  }

  BColor fromColor(Color c) {
    return BColor(rgba: c.red | c.green << 8 | c.blue << 16 | c.alpha << 24);
  }
}
